'This program fetches data from the quandl website.  
'It uses the @UIDIALOG functions to provide an interactive way of providing information on which stocks to retreive, as well as some other options.

'logmode +addin

'--------------------------------------------------------Defaults---------------------------------------------------------------------------
!doGUI	=	1		'bool for whether to show the dialog or not

%start	=	"2000/01/01"		'Start date of data to fetch.  Set at an arbitrarily early date. Note that when the actual data are fetched, the workfile start date will be the minimum of the earliest date for which data are available, or the value of %start
%end	=	@datestr(@now,"yyyy/MM/DD")	'end date of data to fetch.  Set at today's date, using the @now function.

%vars = ""


!dopage	=	1		'bool for whether to open a new workfile/page or not
%importops = """Create new workfile"""	'get information on current workfile and page name
%wfname = @wfname		
if @len(%wfname)>0 then		
	%pname = @pagename	
	%pfreq = @pagefreq
	if %pfreq<>"U" then
		%temp = "Import into page "
		if @upper(%wfname) <> "UNTITLED" then
			if @len(%wfname)>5 then
				%temp = %temp + @mid(%wfname,1,5) + "...\"				
			else
				%temp = %temp + %wfname + "\"					
			endif
		endif
		if @upper(%pname) <> "UNTITLED" then
			if @len(%pname)>5 then
				%temp = %temp + @mid(%pname,1,5) + "..."
			else
				%temp = %temp + %pname					
			endif
		endif
		%importops = %importops  + " ""Create new page""" + " """ + %temp + """"
		%start =  @datestr(@dateval(@otod(1)),"yyyy/MM/DD")
		%end = @datestr(@dateval(@otod(@obsrange)),"yyyy/MM/DD")
	else
		%importops = %importops  + " ""Create new page"""		
	endif
endif


'------------------------------------------------------Command Line--------------------------------------------------------------------------
if @len(%args) then
	%vars = %args
	!dogui = 0
endif

if @len(@option(1))>0 then
	%temp = @equaloption("start")
	if @len(%temp)>0 then
		%start = %temp
	endif
	%temp = @equaloption("end")
	if @len(%temp)>0 then
		%end = %temp
	endif
	%temp = @equaloption("i")
	if @len(%temp)>0 then
		!dopage = @val(%temp)
	endif
endif


'------------------------------------------------------Put up Dialog------------------------------------------------------------------------------
'Note that I am using the line continuation character " _ "    to break up this single command into multiple lines.  This makes it much easier to read.
if !dogui = 1 then
	!result = @uidialog( _
		"caption", "Get Quandl Data", _
		"edit",%vars,"Enter the Quandl Code (codes can be found on the variable's page on the Quandl website)", 250, _
		"text","Enter a start and end date. Note that the dates may be truncated due to data availability", _
		"edit",%start,"Start date (yyyy/mm/dd)", _
		"edit",%end,"End date (yyyy/mm/dd)", _
		"list", !dopage, "Import choice", %importops _
		)
	'check if the user hit cancel on the dialog, if they did, stop the program.
	if !result=-1 then
		stop
	endif
endif
	
'check that the user entered at least one stock symbol.
if @length(%vars)=0 then 
	@uiprompt("You must enter at least one Quandl code")
	stop
endif

if @wcount(%vars)>1 then 
	@uiprompt("Due to Quandl API restrictions, only one code may be entered at a time")
	stop
endif

'-------------------------------------------------------------Option parsing----------------------------------------------------------------------

%datestart = @datestr(@dateval(%start), "YYYY-MM-DD")
%dateend = @datestr(@dateval(%end), "YYYY-MM-DD")


'----------------------------------------------------------------------------Fetch the data------------------------------------------------------------------------------
setmaxerrs 100
%var = @word(%vars,1)
%url = "http://www.quandl.com/api/v1/datasets/" + @upper(%var) + ".csv" + "?trim_start=" + %datestart + "&trim_end=" + %dateend

%pn = "quandl"

if !dopage = 1 then
	wfopen(wf=quandl, page=%pn, type=txt) {%url}        
endif
if !dopage = 2 then
	%pn = @getnextname(%pname)
	import(wf={%wfname}, page=%pn, type=txt) {%url}  
endif
if !dopage = 3 then
	'check: var name availability
	%temp = %var + "*"
	if @wcount(@wlookup(%temp))>0 then
		!result = @uidialog("text",%var + "_*  is already exist.","edit",%tvar,"Enter a different stock name")
		%var = %tvar	
		if @isvalidname(%var) = 0 then
			%var= @makevalidname(%var)
		endif
		if !result = -1 then
			stop
		endif										
	endif	
	'check: frequency conversion		
	!higher = 0
	call higherorlower(!higher, %freq, @pagefreq)
	if !dogui = 1 then
		call getfreqconv(%opt, !higher,%var)
	endif
	import(c={%opt}, type=txt) {%url} 
endif

stop  'note the below code supports fetching more than one series, but since Quandl doesn't give good metadata it is impossible to get working, since you'll have name clashes.

'If more than one variable was entered, loop through them one at a time, importing them into the workfile that was created above.
if @wcount(%vars) > 1 then
for !i=2 to @wcount(%vars)
	%var = @word(%vars,!i)
	%url = "http://www.quandl.com/api/v1/datasets/" + @upper(%var) + ".csv" + "?trim_start=" + %datestart + "&trim_end=" + %dateend
	if @isvalidname(%var) = 0 then
		%var = @makevalidname(%var)
	endif	
	%temp = %var + "_" 
	if @wcount(@wlookup(%temp))>0 then
		!result = @uidialog("text",%var + "_*  is already exist.","edit",%tvar,"Enter a different variable name")
		%var = %tvar	
		if @isvalidname(%var) = 0 then
			%var = @makevalidname(%var)
		endif
		if !result = -1 then
			stop
		endif										
	endif			
	!higher = 0
	call higherorlower(!higher, %freq, @pagefreq)
	if !dogui = 1 then
		call getfreqconv(%opt, !higher,%var)
	endif
	import(resize,c={%opt}, type=txt) {%url} 
next
endif



'----------------------------Subroutines for controlling the frequency differences------------------------------------

subroutine higherorlower(scalar !higher, string %sourcefreq, string %destfreq)
'returns:	1	= if sourcefreq is higher frequency than destfreq
' 			0	= if sourcefreq is lower frequency than destfreq
' 			-1	= if they're the same.
	%freqs = "D5 D7 W T F BM M Q S A"
	%nums = "9 9 8 7 6 5 4 3 2 1"
	
	!source = @wfind(%freqs, @upper(%sourcefreq))
	!source = @val(@word(%nums, !source))
	
	!dest= @wfind(%freqs, %destfreq)
	!dest= @val(@word(%nums, !dest))
	
	!higher = -1
	if !source > !dest then 
		!higher = 1
	endif
	if !source < !dest then
		!higher = 0
	endif
endsub

subroutine getfreqconv(string %opt, scalar !higher, string %name)
	%higherlabs = """Average"" ""Sum"" ""First"" ""Last"" ""Max"" ""Min"" ""Average with NAs"" ""Sum with NAs"" ""First with NAs"" ""Last with NAs"" ""Max with NAs"" ""Min with NAs"""
	%higheropts = "a s f l x m an sn fn ln xn mn"
	
	%lowerlabs = """Constant match average"" ""Constant match sum"" ""Quadratic match average"" ""Quadratic match sum"" ""Linear match last"" ""Cubic match last"""
	%loweropts = "r d q t i c"
	
	!opt = 1
	%temp =  "Select frequency conversion method for " + %name
	if !higher = 1 then
		!result = @uidialog("list",!opt,%temp,%higherlabs)	
		%opt = @word(%higheropts, !opt)
	endif
	if !higher = 0 then
		!result = @uidialog("list",!opt,%temp,%lowerlabs)
		%opt = @word(%loweropts, !opt)
	endif
	if !result=-1 then
		stop
	endif			
endsub


